export class Employee {
  constructor(
    public id: number,
    public firstName: string,
    public lastName: string,
    public explicitPermissions: EmployeePermission[],
    public permissions: EmployeePermission[],
    public group?: EmployeeGroup,
    public lastLoginTime?: Date
  ) {
  }
}

export class EmployeeGroup {
  constructor(
    public id: number,
    public name: string,
    public permissions: EmployeePermission[]
  ) {
  }
}

export enum EmployeePermission {
  VIEW_RECIPES = 'VIEW_RECIPES',
  VIEW_USERS = 'VIEW_USERS',
  VIEW_CATALOG = 'VIEW_CATALOG',

  EDIT_RECIPES_PUBLIC_DATA = 'EDIT_RECIPES_PUBLIC_DATA',
  EDIT_RECIPES = 'EDIT_RECIPES',
  EDIT_MATERIALS = 'EDIT_MATERIALS',
  EDIT_MATERIAL_TYPES = 'EDIT_MATERIAL_TYPES',
  EDIT_COMPANIES = 'EDIT_COMPANIES',
  EDIT_USERS = 'EDIT_USERS',
  EDIT_CATALOG = 'EDIT_CATALOG',

  REMOVE_RECIPES = 'REMOVE_RECIPES',
  REMOVE_MATERIALS = 'REMOVE_MATERIALS',
  REMOVE_MATERIAL_TYPES = 'REMOVE_MATERIAL_TYPES',
  REMOVE_COMPANIES = 'REMOVE_COMPANIES',
  REMOVE_USERS = 'REMOVE_USERS',
  REMOVE_CATALOG = 'REMOVE_CATALOG',

  PRINT_MIXES = 'PRINT_MIXES',
  ADD_TO_INVENTORY = 'ADD_TO_INVENTORY',
  DEDUCT_FROM_INVENTORY = 'DEDUCT_FROM_INVENTORY',
  GENERATE_TOUCH_UP_KIT = 'GENERATE_TOUCH_UP_KIT',

  ADMIN = 'ADMIN'
}

export const mapped_permissions = {
  view: [
    {permission: EmployeePermission.VIEW_RECIPES, description: 'Voir les recettes', impliedPermissions: []},
    {permission: EmployeePermission.VIEW_CATALOG, description: 'Voir le catalogue', impliedPermissions: []},
    {permission: EmployeePermission.VIEW_USERS, description: 'Voir les utilisateurs', impliedPermissions: []},
  ],
  edit: [{
    permission: EmployeePermission.EDIT_RECIPES_PUBLIC_DATA,
    description: 'Modifier les données publiques des recettes',
    impliedPermissions: [EmployeePermission.VIEW_RECIPES]
  }, {
    permission: EmployeePermission.EDIT_RECIPES,
    description: 'Modifier les recettes',
    impliedPermissions: [EmployeePermission.EDIT_RECIPES_PUBLIC_DATA]
  }, {
    permission: EmployeePermission.EDIT_MATERIALS,
    description: 'Modifier les produits',
    impliedPermissions: [EmployeePermission.VIEW_CATALOG]
  }, {
    permission: EmployeePermission.EDIT_MATERIAL_TYPES,
    description: 'Modifier les types de produit',
    impliedPermissions: [EmployeePermission.VIEW_CATALOG]
  }, {
    permission: EmployeePermission.EDIT_COMPANIES,
    description: 'Modifier les bannières',
    impliedPermissions: [EmployeePermission.VIEW_CATALOG]
  }, {
    permission: EmployeePermission.EDIT_CATALOG,
    description: 'Modifier le catalogue',
    impliedPermissions: [EmployeePermission.EDIT_MATERIALS, EmployeePermission.EDIT_MATERIAL_TYPES, EmployeePermission.EDIT_COMPANIES]
  }, {
    permission: EmployeePermission.EDIT_USERS,
    description: 'Modifier les utilisateurs',
    impliedPermissions: [EmployeePermission.VIEW_USERS]
  }],
  remove: [{
    permission: EmployeePermission.REMOVE_RECIPES,
    description: 'Supprimer des recettes',
    impliedPermissions: [EmployeePermission.EDIT_RECIPES]
  }, {
    permission: EmployeePermission.REMOVE_MATERIALS,
    description: 'Supprimer des produits',
    impliedPermissions: [EmployeePermission.EDIT_MATERIALS]
  }, {
    permission: EmployeePermission.REMOVE_MATERIAL_TYPES,
    description: 'Supprimer des types de produit',
    impliedPermissions: [EmployeePermission.EDIT_MATERIAL_TYPES]
  }, {
    permission: EmployeePermission.REMOVE_COMPANIES,
    description: 'Supprimer des bannières',
    impliedPermissions: [EmployeePermission.EDIT_COMPANIES]
  }, {
    permission: EmployeePermission.REMOVE_CATALOG,
    description: 'Supprimer dans le catalogue',
    impliedPermissions: [EmployeePermission.REMOVE_MATERIALS, EmployeePermission.REMOVE_MATERIAL_TYPES, EmployeePermission.REMOVE_COMPANIES]
  }, {
    permission: EmployeePermission.REMOVE_USERS,
    description: 'Supprimer des utilisateurs',
    impliedPermissions: [EmployeePermission.EDIT_USERS]
  }],
  other: [{
    permission: EmployeePermission.PRINT_MIXES,
    description: 'Imprimer les mélanges (bPac)',
    impliedPermissions: [EmployeePermission.VIEW_RECIPES]
  }, {
    permission: EmployeePermission.ADD_TO_INVENTORY,
    description: 'Ajouter dans l\'inventaire',
    impliedPermissions: [EmployeePermission.VIEW_CATALOG]
  }, {
    permission: EmployeePermission.DEDUCT_FROM_INVENTORY,
    description: 'Déduire dans l\'inventaire',
    impliedPermissions: [EmployeePermission.VIEW_RECIPES]
  }, {
    permission: EmployeePermission.ADMIN,
    description: 'Administrateur',
    impliedPermissions: [EmployeePermission.EDIT_CATALOG, EmployeePermission.REMOVE_RECIPES, EmployeePermission.REMOVE_USERS, EmployeePermission.REMOVE_CATALOG, EmployeePermission.PRINT_MIXES, EmployeePermission.ADD_TO_INVENTORY, EmployeePermission.DEDUCT_FROM_INVENTORY]
  }]
}
